﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=8.1
@EndOfDesignText@
Sub Class_Globals
	Private xui As XUI
	Private mPanel As B4XView
	Type BSTNode (Value As Int, Left As BSTNode, Right As BSTNode, Parent As BSTNode)
	Private Root As BSTNode
	Private Empty As BSTNode
	Private cvs As B4XCanvas
End Sub

Public Sub Initialize (Panel As B4XView)
	mPanel = Panel
	cvs.Initialize(mPanel)
	Root = Empty
End Sub

Public Sub Add (Value As Int)
	Dim NewNode As BSTNode = CreateBSTNode(Value, Empty, Empty)
	If Root.IsInitialized = False Then
		Root = NewNode
	Else
		AddHelper(NewNode, Root)
	End If
	DrawTree
End Sub

Private Sub AddHelper (NewNode As BSTNode, N As BSTNode)
	If NewNode.Value > N.Value Then
		If N.Right.IsInitialized = False Then
			N.Right = NewNode
			NewNode.Parent = N
		Else
			AddHelper(NewNode, N.Right)
		End If
	Else
		If N.Left.IsInitialized = False Then
			N.Left = NewNode
			NewNode.Parent = N
		Else
			AddHelper(NewNode, N.Left)
		End If
	End If
End Sub

Public Sub GetAllValuesSorted As List
	Dim res As List
	res.Initialize
	GetAllValuesHelper(Root, res)
	Return res
End Sub

Public Sub Delete (Value As Int)
	Dim NodeToDelete As BSTNode = FindNode(Root, Value)
	If NodeToDelete.IsInitialized = False Then Return
	Dim Parent As BSTNode = NodeToDelete.Parent
	Dim IsRoot As Boolean = Parent.IsInitialized = False
	Dim NewChild As BSTNode
	If NodeToDelete.Left.IsInitialized = False And NodeToDelete.Right.IsInitialized = False Then
		'no children. just remove
		NewChild = Empty
	Else if NodeToDelete.Left.IsInitialized <> NodeToDelete.Right.IsInitialized Then
		'single child
		Dim NewChild As BSTNode = NodeToDelete.Left
		If NewChild.IsInitialized = False Then NewChild = NodeToDelete.Right
	Else
		'two children. Replace with the minimum node on the right side
		Dim NewChild As BSTNode = FindMin(NodeToDelete.Right)
		If NewChild <> NodeToDelete.Right Then
			NewChild.Parent.Left = Empty
			NewChild.Right = NodeToDelete.Right
			NodeToDelete.Right.Parent = NewChild
		End If
		NewChild.Left = NodeToDelete.Left
		NodeToDelete.Left.Parent = NewChild
	End If
	If IsRoot Then
		Root = NewChild
		Root.Parent = Empty
	Else
		NewChild.Parent = NodeToDelete.Parent
		If NodeToDelete.Value > Parent.Value Then
			Parent.Right = NewChild
		Else
			Parent.Left = NewChild
		End If
	End If
	DrawTree
End Sub

Private Sub FindMin(n As BSTNode) As BSTNode
	If n.Left.IsInitialized Then Return FindMin(n.Left) Else Return n
End Sub

Private Sub FindNode (n As BSTNode, Value As Int) As BSTNode
	If n.IsInitialized = False Then Return Empty
	If Value = n.Value Then Return n
	If Value < n.Value Then Return FindNode(n.Left, Value)
	Return FindNode(n.Right, Value)
End Sub

Private Sub GetAllValuesHelper (n As BSTNode, Res As List)
	If n.IsInitialized = False Then Return
	GetAllValuesHelper(n.Left, Res)
	Res.Add(n.Value)
	GetAllValuesHelper(n.Right, Res)
End Sub

Private Sub DrawTree
	cvs.ClearRect(cvs.TargetRect)
	DrawTreeHelper(Root, 0, 0, mPanel.Width / 2, 20dip, 0)
End Sub

Private Sub DrawTreeHelper (n As BSTNode, PrevX As Int, PrevY As Int, X As Int, Y As Int, Depth As Int)
	If n.IsInitialized = False Then Return
	Dim DeltaX As Int = mPanel.Width / (Power(2, Depth + 1) + 0.5)
	DrawTreeHelper(n.Left, X, Y, X - DeltaX / 2, Y + 80dip, Depth + 1)
	DrawTreeHelper(n.Right, X, Y, X + DeltaX / 2, Y + 80dip, Depth + 1)
	If Depth > 0 Then
		cvs.DrawLine(PrevX, PrevY, X, Y, xui.Color_Black, 2dip)
	End If
	cvs.DrawCircle(X, Y, 15dip, xui.Color_Black, False, 2dip)
	cvs.DrawCircle(X, Y, 15dip, xui.Color_White, True, 2dip)
	cvs.DrawText(n.Value, X, Y + 7dip, xui.CreateDefaultFont(20), xui.Color_Black, "CENTER")
End Sub

Private Sub CreateBSTNode (Value As Int, Left As BSTNode, Right As BSTNode) As BSTNode
	Dim t1 As BSTNode
	t1.Initialize
	t1.Value = Value
	t1.Left = Left
	t1.Right = Right
	Return t1
End Sub